﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using System.Threading;

namespace LED_Display
{
	public delegate void ChangedEventHandler(object sender, EventArgs e);
	
	public partial class LED_Display_Control : UserControl
	{
		#region Vars
		private Graphics display;
		private bool[,] _Grid;
		private Color _oneColor = Color.Red;
		private Color _zeroColor = Color.Black;
		private int _LED_Diameter = 0;
		private int LEDsWide, LEDsHigh;

		private bool round = true;
		private List<Point> _mousepos;
		private bool _ReadOnly = false;
		private int _Text_Size=8;
		
		public event ChangedEventHandler Changed;
		#endregion

		#region Events and Constructor
		public LED_Display_Control()
		{
			InitializeComponent();
		}

		private void LED_Grid_Display_Load(object sender, EventArgs e)
		{
			if (_LED_Diameter == 0) _LED_Diameter = 50;
			resetDisplay();
		}

		private void LED_Grid_Display_Paint(object sender, PaintEventArgs e)
		{
			display = e.Graphics;

			for (int y = 0; y < LEDsHigh; y++)for (int x = 0; x < LEDsWide; x++)
			{
				Rectangle rect=new Rectangle(x * _LED_Diameter, y * _LED_Diameter, _LED_Diameter, _LED_Diameter);
				if(round) display.FillEllipse(getBrush(x, y), rect);
				else display.FillRectangle(getBrush(x, y), rect);
				int pos=LEDsWide*LEDsHigh-(LEDsHigh-y-1)*LEDsWide-x-1;
				
				display.DrawString(pos.ToString(),new Font("Arial",_Text_Size),getInvBrush(x, y),rect.Left,rect.Top);
			}
		}
		
		private SolidBrush getBrush(int x,int y){
			if(_Grid[x, y]) return new SolidBrush(_oneColor);
			else return new SolidBrush(_zeroColor);
		}
		
		private SolidBrush getInvBrush(int x,int y){
			if(!_Grid[x, y]) return new SolidBrush(_oneColor);
			else return new SolidBrush(_zeroColor);
		}
		
		private void LED_Grid_Display_Resize(object sender, EventArgs e)
		{
			if (this.Width % _LED_Diameter != 0)
				this.Size = new Size((int)((this.Width / _LED_Diameter) * _LED_Diameter) + 1, this.Height);

			if (this.Height % _LED_Diameter != 0)
				this.Size = new Size(this.Width, (int)((this.Height / _LED_Diameter) * _LED_Diameter) + 1);

			resetDisplay();
		}
		#endregion

		#region Methods
		/// <summary>
		/// Will reset the display.
		/// <param name="clear">Do you wish to clear the grid too?</param>
		/// </summary>
		private void resetDisplay()
		{
			LEDsWide = (int)(this.Width / _LED_Diameter);
			LEDsHigh = (int)(this.Height / _LED_Diameter);

			_Grid = new bool[LEDsWide, LEDsHigh];

			setAllLEDs(false);
		}

		/// <summary>
		/// Check if a location exists on the grid
		/// <param name="location">The location of the LED (using LEDs wide by LEDs high)</param>
		/// </summary>
		public bool isLocValid(Point location)
		{
			return location.X >= 0 && location.Y >= 0 &&
				_Grid.GetLength(0) > location.X && _Grid.GetLength(1) > location.Y;
		}

		/// <summary>
		/// Set the color of only one LED to a certain color.
		/// Will check if location is valid using isValid(Point location);
		/// </summary>
		/// <param name="location">The location of the LED on the grid</param>
		/// <param name="color">The color which to make the LED specified</param>
		public void setLEDAt(Point location, bool State)
		{
			if (isLocValid(location))
				_Grid[location.X, location.Y] = State;
		}

		/// <summary>
		/// Set the color of only one LED to a certain color.
		/// Will check if location is valid using isValid(Point location);
		/// </summary>
		/// <param name="location">The location of the LED on the grid</param>
		/// <param name="color">The color which to make the LED specified</param>
		public void invertLEDAt(Point location)
		{
			if (isLocValid(location))
				_Grid[location.X, location.Y] = !_Grid[location.X, location.Y];
			if (Changed != null) Changed(this, EventArgs.Empty);
		}

		
		/// <summary>
		/// Place the array of colors on the display.
		/// Will check if locations are valid
		/// </summary>
		/// <param name="grid">The array containg the data</param>
		public void placeColorArray(bool[,] grid)
		{
			if (grid == null)return;

			if (grid.GetLength(0) <= _Grid.GetLength(0) && grid.GetLength(1) <= _Grid.GetLength(1)){
				for (int y = 0; y < grid.GetLength(1); y++)
					for (int x = 0; x < grid.GetLength(0); x++)
						setLEDAt(new Point(x, y), grid[x, y]);
			}
		}

		/// <summary>
		/// Get the color of only one LED.
		/// Will NOT check if location is valid using isValid(Point location);
		/// </summary>
		/// <param name="location">The location of the LED on the grid</param>
		public bool getLEDAt(Point location){
			return _Grid[location.X, location.Y];
		}
		
		/// <summary>
		/// Get the color of only one LED.
		/// Will NOT check if location is valid using isValid(Point location);
		/// </summary>
		/// <param name="location">The location of the LED on the grid</param>
		public string getLEDString(){
			string result="";
			for (int y = 0; y < LEDsHigh; y++) for (int x = LEDsWide-1; x >= 0; x--){
				if (_Grid[x, y]) result+="1";
				else result+="0";
			}
			return result;
		}
		
		/// <summary>
		/// Get the color of only one LED.
		/// Will NOT check if location is valid using isValid(Point location);
		/// </summary>
		/// <param name="location">The location of the LED on the grid</param>
		public void setAllLEDs(string Val){
			for (int i = 0; i < Val.Length; i++){
				int y=LEDsHigh-1-i/LEDsWide;
				int x=i%LEDsWide;
				switch (Val[i]){
						case 'H': _Grid[x, y]=true; break;
						case 'L': _Grid[x, y]=false;break;
						case '1': _Grid[x, y]=true; break;
						case '0': _Grid[x, y]=false;break;
						case 'X': _Grid[x, y]=!_Grid[x, y];break;
						case 'U': _Grid[x, y]=!_Grid[x, y];break;
				}
				Invalidate();
			}
		}
		

		/// <summary>
		/// Set all the LEDs of a the display to a color.
		/// </summary>
		/// <param name="color">The color to make the all of the LEDs</param>
		public void setAllLEDs(bool State)
		{
			for (int y = 0; y < LEDsHigh; y++) for (int x = 0; x < LEDsWide; x++)
				_Grid[x, y] = State;
		}

		/// <summary>
		/// Set all the LEDs of a the grid to a random color (each one is random).
		/// </summary>
		public void setAllLEDsToRandom()
		{
			Random Val=new Random(DateTime.Now.Millisecond);
			for (int y = 0; y < LEDsHigh; y++) for (int x = 0; x < LEDsWide; x++){
				_Grid[x, y] = (bool)(Val.NextDouble()>=0.5);
			}
		}
		#endregion

		#region Properties
		[CategoryAttribute("Grid Properties"),DescriptionAttribute("Get the array or grid that contains the color data.")]
		public bool[,] Grid{
			get{
				bool[,] colors = new bool[LEDsWide, LEDsHigh];
				for (int y = 0; y < LEDsHigh; y++)
					for (int x = 0; x < LEDsWide; x++)
						colors[x, y] = _Grid[x, y];
				return colors;
			}
		}
		
		[CategoryAttribute("Grid Properties"),DescriptionAttribute("Get or set the radius of the LEDs")]
		public int LED_Radius{
			get { return _LED_Diameter / 2; }
			set {
				if (value == 0) value = 1;
				_LED_Diameter = value * 2;
				resetDisplay();
			}
		}

		[CategoryAttribute("Grid Properties"),DescriptionAttribute("Get or set the text size")]
		public int Text_Size{
			get { return _Text_Size;}
			set {_Text_Size=value;}
		}		
		
		[CategoryAttribute("Grid Properties"),DescriptionAttribute("Get the size of the display (LEDs wide by LEDs high)")]
		public Size LED_Grid_Size{
			get { return new Size(LEDsWide, LEDsHigh); }
		}

		[CategoryAttribute("Grid Properties"),DescriptionAttribute("Set true to draw round LEDs. Set false to draw square LEDs")]
		public bool Round_LEDs{
			get { return round; }
			set { round = value; }
		}
		
		[CategoryAttribute("Grid Properties"),DescriptionAttribute("Enable/Disable Mouse Input")]
		public bool ReadOnly{
			get{return _ReadOnly;}
			set{_ReadOnly=value;}
		}

		[CategoryAttribute("Grid Properties"),DescriptionAttribute("Set the starting color of all the LEDs")]
		public Color Start_Color{
			get { return _oneColor; }
			set{
				_oneColor = value;
				resetDisplay();
			}
		}
		#endregion
		
		void LED_Display_ControlMouseDown(object sender, MouseEventArgs e){
			if (_ReadOnly) return;
			Point curpos = new Point((int)(e.X / (LED_Radius * 2)), (int)(e.Y / (LED_Radius * 2)));
			_mousepos=new List<Point>();
			_mousepos.Add(curpos);
			
			invertLEDAt(curpos);
			Refresh();
		}
		
		void LED_Display_ControlMouseMove(object sender, MouseEventArgs e){
			if (_ReadOnly) return;
			
			Point curpos = new Point((int)(e.X / (LED_Radius * 2)), (int)(e.Y / (LED_Radius * 2)));
			
			if(e.Button==MouseButtons.Left){
				if (!_mousepos.Contains(curpos)){
					invertLEDAt(curpos);
					_mousepos.Add(curpos);
				}
			}
			Refresh();
		}
	}
}
